#!/share/apps/perl/bin/perl
#make a plot and a test summary of the sequences around insertion positions

use strict;
use warnings;
use Bio::SeqIO;
use Data::Dumper;
use Getopt::Long;

my ($genome,$outputfile,$up,$down,$number);
GetOptions(
	'genome|g=s' => \$genome,
	'out|o=s' 	=>\$outputfile,
	'up|u=i' => \$up,
	'down|d=i' =>\$down,
	'number|n=i' =>\$number
);


my $use = "\nUSE: cat insert-positions.txt | $0 --genome genome.fasta  --out outputfile.txt  [--up int] [--down int] [--number int]
--number is the number for process (default=all)
--up is how many nt upstream of the position we analyse (default=0)
--down is how many nt downstream of the position we analyse (default=6)
insert-positions.txt must have chr pos strand (space delim)
\n\n";

#defaults
die $use unless ($genome and $outputfile);
$up ||= 0;
$down ||= 6;
$number ||= 99e9999;

#load the seq
my $inseq = Bio::SeqIO->new(-file => "$genome",-format => 'fasta');
my $outseq = Bio::SeqIO->new(-fh => \*STDOUT,-format => 'fasta');	
my %seq=();
my $profile={};
my %fits=();

#$profile->{fwd}{0}{A}=$proportion;
#$profile->{fwd}{0}{C}=$proportion;
 
while (my $s = $inseq->next_seq) {
	my $chr = $s->id;
	$seq{$chr} = $s;
}


#header
open(OUT,">$outputfile");
print OUT "#script $0
#up $up
#down $down
#number $number\n";


#read in the STDIN & extract seqs
my $counter=0;
while(<STDIN>){
	my $sequence;
	if ($_=~/#/){print OUT $_; next};
	if ($_=~/strand/){next};
	my ($chr,$pos,$strand) = split(' ',$_);
	$counter++;
	if ($counter > $number){last};
	if ($strand eq "+"){
		
		#define the sequence positions
		my $start = $pos - $up;
		my $end = $pos + $down;
		
		#catch any sequences that are out of bounds, and then make the seq
		if ($start < 1){$start=1};
		if ($end > $seq{$chr}->length){$end=$seq{$chr}->length};
		$sequence = $seq{$chr}->subseq($start,$end);
	}
	if ($strand eq "-"){

		#define the sequence positions
		my $end = $pos + $up;
		my $start = $pos - $down;

		#catch any sequences that are out of bounds, and then make the seq
		if ($start < 1){$start=1};
		if ($end > $seq{$chr}->length){$end=$seq{$chr}->length};
		my $sub = $seq{$chr}->subseq($start,$end);
		my $subob = Bio::Seq->new(-display_id=>'sub',-seq =>$sub); 
		$sequence = $subob->revcom->seq();
	}
	$sequence = lc $sequence;
	$sequence =~ s/t/T/g;$sequence =~ s/a/A/g;
	#print "$sequence\t$strand\n";
	add_to_profile($sequence,$strand);

}
warn "#Procesed $number lines\n";
#print Dumper \$profile;

#how many fit profile:
$fits{no}||= 0;
$fits{yes}||= 0;
my $proportion_fitting = $fits{yes}/($fits{yes}+$fits{no});

#print out the profile to a file
print OUT "##proportion_fitting $proportion_fitting\n";
print OUT join("\t",qw(position A T G C total)),"\n";
for my $p (sort {$a <=> $b} keys %{$profile}){
	print OUT "$p";
	for my $base (qw(A T G C total)){
		print OUT "\t",$profile->{$p}{$base};
	}
	print OUT "\n";
}
close OUT;

#make the plot and show it:
system("cat analyse-seq-profile-plot.r | R --vanilla --slave --args $outputfile");

#show the result
print "Output file is:$outputfile\n\n";
system("cat $outputfile");


#subs
sub add_to_profile{
	my ($seq,$strand) = @_;
	
	my @a = split(//,$seq);
	#check if it has TnnnnA
	if (($a[0-$up+1] eq 'T') and ($a[5-$up+1] eq 'A')){$fits{yes}++} else {$fits{no}++};
	
	for my $i (0..scalar @a -1){
		my $j = $i - $up;
		$profile->{$j}{uc $a[$i]}++;
		$profile->{$j}{total}++;
	}
}

exit;
